#!/bin/sh

#---------------------------------
# shell functions to implement
# config file items.
#

set_variable() {
   set -x
   cmd="`echo $1 | sed -e 's/=\(.*\)/=\"\1\"/'`"
   eval $cmd
}

pre_install_command() {
   set -x
   eval "$1"
}

post_install_command() {
   set -x
   eval "$1"
}

check_space() {
   set -x
    dir=$1

    #
    # look for an existing containing directory to actually check
    #
    while [ ! -d $dir ]
    do
	dir=`echo $dir | sed -e 's;\(.*\)/.*;\1;' -e 's;^$;/;' `
    done

    need=$2
    heading="`df -k $dir | head -1`"
    body="`df -k $dir | sed -e 1d`"
    set : $heading
    avcol=0
    mtcol=0
    col=1
    for f in $*
    do
        case $f in
        [Aa]vail*) avcol=$col;;
        [Mm]ount*) mtcol=$col;;
        esac
        col=`expr $col + 1`
    done
    set : $body
    eval avail=\$$avcol
    eval mount=\$$mtcol
    mountvar="mv`echo $mount | sed -e 's/[^a-z]/_/g'`"
    eval "need=\`expr $need + \${$mountvar:-0}\`"
    eval "$mountvar=$need"
    if [ "$avail" -lt "$need" ]
    then
        echo "Insufficient space in $mountvar partition."
        echo "$mountvar has ${avail}k free, but ${need}k needed"
        return 1
    else
        return 0
    fi
}

download_file() {
   set -x
    perl $PERL_DIR/bin/lwp-download "$@"
}

create_db() {
   set -x
   mkdir -p $1
   mkdir $1/.updfiles
   mkdir $1/.upsfiles


   if [ x$2 = x- ]
   then
        base="`echo $1 | sed -e 's;\(.*\)/.*;\1;'`"
	cp $UPS_DIR/ups/dbconfig.template $1/.upsfiles/dbconfig
	perl -pi.template -e "s{/fnal/ups/db}{$1}go; s{/fnal/ups}{$base}go;" $1/.upsfiles/dbconfig
   else
	cp $2 $1/.upsfiles/dbconfig
   fi

   if [ x$3 = x- ]
   then
	cp $UPD_DIR/ups/updconfig.template $1/.updfiles/updconfig
   else
	cp $3  $1/.updfiles/updconfig
   fi

   # fix ownerships
   if [ -w / -a x$4 != x- ]
   then
     chown -R $4 $1
     chown $4 $1/..
   fi

   # 
   # if this database has a SETUPS_DIR
   # make it and a upsdb_list therein.
   #
   line=`grep SETUPS_DIR $1/.upsfiles/dbconfig | sed -e 's/ //g' `

   case "x$line" in
   x\#*) ;;
   x*=*)
       eval $line
       mkdir -p ${SETUPS_DIR}
       #
       # make the upsdb list
       #
       for d in $upsdb_list
       do
           echo $d >> ${SETUPS_DIR}/upsdb_list
       done
       if [ x$4 != x- ]
       then
         chown -R $4 ${SETUPS_DIR}
       fi
       ;;
   *) ;;
   esac

   # old ups bug workaround, kept for good luck
   mkdir $1/xxx
}

# creates an account in the local password file, happily ignoring

create_user() {
   set -x
   if [ -w /etc/passwd ]
   then
       if [ `grep "^$1:" /etc/passwd | wc -l` = 0 ]
       then
	    echo "$1:x:$2:$3:$1 account:$4:/bin/sh" >> /etc/passwd
	    if [ -f /etc/shadow ]
	    then
		echo "$1:NP:6445::::::" >> /etc/shadow
	    fi
       fi
   else
       echo "Unable to create user $1"
   fi
}

install_coreFUE() {
   set -x
    db=$1
    user="${2:-products}"
    shift
    if [ $# != 0 ]; then shift; fi
    pre="`upd depend -h file://localhost$BOOTUPS/db coreFUE -K+`"
    install_as $user coreFUE -h file://localhost$BOOTUPS/db -z $db "$@"
    post="`ups depend -z $db coreFUE -K+`"
    if [ "$pre" != "$post" ]
    then
	exit 1
    fi
}

install_as() {
   set -x
   id=$1
   shift

   if [ x$setupfile = x ]
   then
       setupfile=`ups setup -V ups, -V upd`
   fi

   if [ -w / -a x$id != x- -a `grep "^$id:" /etc/passwd 2>/dev/null | wc -l` != 0 ]
   then
	su $id -c "/bin/sh -c '. $setupfile; upd install -h ${BOOTHOST:-fnkits.fnal.gov} -z $PRODUCTS -vvv -G -c $*'"
   else
	upd install -h ${BOOTHOST:-fnkits.fnal.gov} -z $PRODUCTS -vvv -G -c $* 
   fi
}

do_ups() {
   set -x
   ups "$@"
}

make_courtesy_links() {
   set -x
   test -d /usr/local/etc || mkdir -p /usr/local/etc
   cd /usr/local/etc
   eval set : `ups list -K setups_dir`
   SETUPS_DIR=$2
   if [ "${SETUPS_DIR}" != /usr/local/etc ]
   then
       for f in setups.sh setups.csh
       do
	    ln -s $SETUPS_DIR/$f $f
       done
   fi
}

# basically, all of the various kinds of config lines are egrep-ed out of the
# config file as executable shell statements, and
# then eval executes them in the current shell.
 
perform_action() {
   set -x
   eval "re=\"\$${1}_re\""
   if [ x"$re" = x ]
   then
       echo "Internal error in stage2.sh: no ${1}_re variable"
   else
       cmds="`egrep \"$re\" $config`"
       if $debug
       then
           echo "$cmds"
       else
           eval "$cmds"
       fi
   fi
}

# --------------------------------------

if [ "$1" = "-d" ]
then
    debug=true
    shift
else
    debug=false
fi

config=$1
BOOTUPS=$2

#
# make sure we really use setups.sh
#
PRODUCTS=$BOOTUPS/db
export PRODUCTS
unset UPS_DIR
unset SETUP_UPS

. $BOOTUPS/etc/setups.sh
setup upd


#---------------------------------------
# Syntax checks...
#

#
# egrep regexps for valid config_file lines
#
	        s='[ 	]'		# whitespace regexp
              var='[a-zA-Z0-9_]+'	# variable name regexp
             user='[-a-zA-Z0-9_]+'	# user name rexexp
          optuser='[-a-zA-Z0-9_]*'	# user name rexexp (optional)
             path='[-$/:.a-zA-Z0-9_]+'	# pathname/URL regexp
          optpath='[-$/:.a-zA-Z0-9_]*'	# optional pathname regexp
              num='[0-9]+'		# number regexp
	   quotes='".*"'		# quoted string regexp

       comment_re="^\#.*$|^$s*$"
	   set_re="^set_variable$s+$var=$quotes$s*\$"
   check_space_re="^check_space$s+$path$s+$num$s*\$"
   create_user_re="^create_user$s+$user$s+$num$s+$num$s+$path$s*\$"
 download_file_re="^download_file$s+$path$s*$optpath$s*\$"
   pre_install_re="^pre_install_command$s+$quotes$s*\$"
     create_db_re="^create_db$s+$path$s+$path$s+$path$s+$user$s*\$"
  install_core_re="^install_coreFUE$s+$path$s*$optuser$s*.*\$"
    install_as_re="^install_as$s+$user$s+.*"
        do_ups_re="^do_ups$s+.*"
  post_install_re="^post_install_command$s+$quotes$s*\$"
courtesy_links_re="^make_courtesy_links$s*\$"

#
# build up disjunction of all valid config file lines
#

valid_re="$comment_re|$set_re|$check_space_re|$download_file_re|$create_user_re"
valid_re="$valid_re|$pre_install_re|$create_db_re|$install_as_re"
valid_re="$valid_re|$install_core_re|$do_ups_re|$post_install_re|$courtesy_links_re"

#
# now egrep can syntax check the file
#

if [ `egrep -vn "$valid_re" $config | wc -l` -gt 0 ]
then
    printf "Your config file, %s, contains the following syntax errors:\n"
    egrep -vn "$valid_re" $config 
    printf "You may correct them, and then run:\n  %s %s %s\nto complete the install" $0 $config $BOOTUPS
    exit 1
fi

echo ok > .register_test
if [ ! -s .register_test ] 
then
    echo "You are not on a registered ftp.fnal.gov host, or ftp.fnal.gov is down"
    exit 1
fi 

#
# implement the config file
#

perform_action set
perform_action check_space
perform_action create_user
perform_action download_file
perform_action pre_install
perform_action create_db

perform_action install_core

# do the rest with the new local db's
PRODUCTS=`echo $upsdb_list | sed -e 's/ /:/g'`
export PRODUCTS

setup ups

perform_action install_as
perform_action do_ups
perform_action post_install
perform_action courtesy_links


